@@ -158,7 +158,7 @@ sub dispatch_to_swagger {
   return Mojo::IOLoop->delay(
     sub {
       my $delay = shift;
-      my $sc = $delay->data->{sc} = $sc_class->new(tx => $c->tx);
+      my $sc = $delay->data->{sc} = $sc_class->new($c);    # clone
       $sc->stash(swagger_operation_spec => $op_info->{spec});
       $sc->$method_ref($input, $delay->begin);
     },
@@ -468,13 +468,20 @@ sub _validate_input {
   for my $p (@{$op_spec->{parameters} || []}) {
     my ($in, $name, $type) = @$p{qw( in name type )};
     my $value
-      = $in eq 'query'    ? $query->param($name)
+      = $in eq 'query'    ? $query->every_param($name)
       : $in eq 'path'     ? $c->stash($name)
       : $in eq 'header'   ? $headers->header($name)
       : $in eq 'body'     ? $c->req->json
-      : $in eq 'formData' ? $body->param($name)
+      : $in eq 'formData' ? $body->every_param($name)
       :                     "Invalid 'in' for parameter $name in schema definition";
 
+    if (ref $p->{items} eq 'HASH' and $p->{collectionFormat}) {
+      $value = _coerce_by_collection_format($value, $p);
+    }
+    elsif (ref $value eq 'ARRAY') {
+      $value = $value->[0];
+    }
+
     if ($type and defined($value //= $p->{default})) {
       if (($type eq 'integer' or $type eq 'number') and $value =~ /^-?\d/) {
         $value += 0;
@@ -482,9 +489,6 @@ sub _validate_input {
       elsif ($type eq 'boolean') {
         $value = (!$value or $value eq 'false') ? Mojo::JSON->false : Mojo::JSON->true;
       }
-      elsif (ref $p->{items} eq 'HASH' and $p->{items}{collectionFormat}) {
-        $value = _coerce_by_collection_format($p->{items}, $value);
-      }
     }
 
     my @e = $self->_validate_value($p, $name => $value);
@@ -505,8 +509,6 @@ sub _validate_value {
   my $schema = {properties => {$name => $p->{schema} || $p}, required => [$p->{required} ? ($name) : ()]};
   my $in = $p->{in};
 
-  # ugly hack
-
   if ($in eq 'body') {
     warn "[Swagger2] Validate $in $name\n" if DEBUG;
     return $self->_validator->validate({$name => $value}, $schema);
@@ -523,14 +525,20 @@ sub _validate_value {
   return;
 }
 
-# copy/paste from JSON::Validator
 sub _coerce_by_collection_format {
-  my ($schema, $data) = @_;
-  my $format = $schema->{collectionFormat};
-  my @data = $format eq 'ssv' ? split / /, $data : $format eq 'tsv' ? split /\t/,
-    $data : $format eq 'pipes' ? split /\|/, $data : split /,/, $data;
+  my ($data, $schema) = @_;
+  my $re = $Swagger2::SchemaValidator::COLLECTION_RE{$schema->{collectionFormat}} || '';
+  my $type = $schema->{items}{type} || '';
+  my @data;
 
-  return [map { $_ + 0 } @data] if $schema->{type} and $schema->{type} =~ m!^(integer|number)$!;
+  if (!$re) {
+    @data = @$data;
+  }
+  elsif (defined($data = $data->[0])) {
+    @data = split /$re/, $data;
+  }
+
+  return [map { $_ + 0 } @data] if $type eq 'integer' or $type eq 'number';
   return \@data;
 }
 
