@@ -11,6 +11,11 @@ sub authenticate {
   return $c->render(json => $config, status => $CODE);
 }
 
+sub collection_format {
+  my ($c, $args, $cb) = @_;
+  $c->$cb($args, $CODE);
+}
+
 sub get_headers {
   my ($c, $args, $cb) = @_;
 
@@ -45,11 +50,6 @@ sub status {
   $c->$cb($resp, $CODE);
 }
 
-sub query_as_array {
-  my ($c, $args, $cb) = @_;
-  $c->$cb($args, $CODE);
-}
-
 sub show_pet_by_id {
   my ($c, $args, $cb) = @_;
   $RES->{id} = $args->{petId};
